/* Langton's Ant
 * 
 * Author: Aaron Weeden, Shodor, 2016
 * 
 * Naming convention:
 *   ALL_CAPS for constants
 *   PascalCase for globals and functions
 *   camelCase for locals
 */

/*******************************************************************************
 * IMPORT LIBRARIES
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*******************************************************************************
 * DEFINE CONSTANTS
 ******************************************************************************/
// Model parameters
#define TIME_STEP_COUNT 1000
#define ANT_COUNT 1
#define CELL_COUNT_X 21
#define CELL_COUNT_Y CELL_COUNT_X
#define RANDOM_SEED 0

// Cell states
#define BLACK 0
#define WHITE 1

// Ant directions
#define UP    0
#define LEFT  1
#define RIGHT 2
#define DOWN  3

/*******************************************************************************
 * DECLARE GLOBAL VARIABLES
 ******************************************************************************/
int CellStates[CELL_COUNT_Y][CELL_COUNT_X];

/*******************************************************************************
 * DEFINE FUNCTIONS
 ******************************************************************************/
// Generate a random integer between [min..max)
int RandBetween(const int min, const int max) {
  return min + (random() % (max - min));
}

/*******************************************************************************
 * DEFINE MAIN FUNCTION
 ******************************************************************************/
int main() {
  // Initialize cell states
  int CellStates[CELL_COUNT_Y][CELL_COUNT_X];
  for (int y = 0; y < CELL_COUNT_Y; y++) {
    for (int x = 0; x < CELL_COUNT_X; x++) {
      CellStates[y][x] = WHITE;
    }
  }

  // Seed the random number generator
  srandom(RANDOM_SEED);

  // Give each ant a position and direction
  int antCellXs[ANT_COUNT];
  int antCellYs[ANT_COUNT];
  int antDirs[ANT_COUNT];
  for (int i = 0; i < ANT_COUNT; i++) {
    antCellXs[i] = RandBetween(0, CELL_COUNT_X);
    antCellYs[i] = RandBetween(0, CELL_COUNT_Y);
    antDirs[i] = LEFT;
  }

  // Start the simulation
  for (int timeStep = 0; timeStep < TIME_STEP_COUNT; timeStep++) {
    // Output
    printf("Time %d\n", timeStep);
    for (int y = CELL_COUNT_Y-1; y >= 0; y--) {
      for (int x = 0; x < CELL_COUNT_X; x++) {
        printf("%d", CellStates[y][x]);
      }
      printf("\n");
    }
    printf("\n");

    // Turn
    for (int i = 0; i < ANT_COUNT; i++) {
      int const isBlack = (CellStates[antCellYs[i]][antCellXs[i]] == BLACK);
      if (       (antDirs[i] == UP    &&  isBlack) ||
                 (antDirs[i] == DOWN  && !isBlack)) {
        antDirs[i] = LEFT;
      } else if ((antDirs[i] == UP    && !isBlack) ||
                 (antDirs[i] == DOWN  &&  isBlack)) {
        antDirs[i] = RIGHT;
      } else if ((antDirs[i] == LEFT  &&  isBlack) ||
                 (antDirs[i] == RIGHT && !isBlack)) {
        antDirs[i] = DOWN;
      } else if ((antDirs[i] == LEFT  && !isBlack) ||
                 (antDirs[i] == RIGHT &&  isBlack)) {
        antDirs[i] = UP;
      }
    }

    // Update cells
    for (int i = 0; i < ANT_COUNT; i++) {
      CellStates[antCellYs[i]][antCellXs[i]] =
        (CellStates[antCellYs[i]][antCellXs[i]] == BLACK) ? WHITE : BLACK;
    }

    // Move
    for (int i = 0; i < ANT_COUNT; i++) {
      switch (antDirs[i]) {
        case UP:
          if (antCellYs[i] == 0) {
            antCellYs[i] = CELL_COUNT_Y-1;
          } else {
            antCellYs[i]--;
          }
          break;
        case LEFT:
          if (antCellXs[i] == 0) {
            antCellXs[i] = CELL_COUNT_X-1;
          } else {
            antCellXs[i]--;
          }
          break;
        case RIGHT:
          if (antCellXs[i] == CELL_COUNT_X-1) {
            antCellXs[i] = 0;
          } else {
            antCellXs[i]++;
          }
          break;
        case DOWN:
          if (antCellYs[i] == CELL_COUNT_Y-1) {
            antCellYs[i] = 0;
          } else {
            antCellYs[i]++;
          }
          break;
      }
    }
  }

  return 0;
}

